
import logging

log				= logging.getLogger("isolate")
log.setLevel( logging.DEBUG )

def method(name=None, fn=None, err=None):
    if fn is None:
        def wrap(f):
            method(f)
            return f
        return wrap

    if name is None:
        name			= fn.__name__
        if name == '<lambda>':
            raise Exception("Lambda given with no method name.  Method won't be callable without a name.")

    gvars			= globals()
    def wrapper(*args):
        nonlocal gvars
        ctx			= gvars.get('isolate/fnctx', {})
        try:
            return fn(ctx, *args)
        except Exception as e:
            log.exception("Method {} failed to execute with arguments: {}".format(name, args))
            if callable(err):
                err(e)
        return None
            
    wrapper.source		= fn
    gvars[name]			= wrapper

def error(fn=None):
    gvars			= globals()
    if fn is None:
        def wrap(f):
            gvars['isolate/errfn']	= f
            return f
        return wrap
    else:
        gvars['isolate/errfn']	= fn

def evaluate(code, context=None, fn_context=None):
    gvars			= globals()
    context			= context or gvars
    gvars['isolate/fnctx']	= fn_context or context
    err				= gvars.get('isolate/errfn')
    try:
        def __eval__(self, __code__):
            log.debug("Evaluating code '{}' in context {}".format(code, self))
            return eval( __code__ )
        return __eval__(context, code)
    except Exception as e:
        if callable(err):
            err(e)
        return None




import logging

from .				import isolate
from .				import Populater

log				= logging.getLogger("tests")
log.setLevel( logging.DEBUG )


def test_isolate_method():
    isolate.method('chin', lambda self, *args: max(*args))

    assert isolate.evaluate('chin(1,2,3)') == 3

def test_isolate_method_decorator():
    @isolate.method()
    def chin(self, *args):
        return max(*args)

    assert isolate.evaluate('chin(1,2,3)') == 3

def test_isolate_method_fail():
    def assert_error(e):
        log.warn("ERROR: {}".format(e))
        assert isinstance(e, Exception)
        
    isolate.method('chin', max, assert_error)

    assert isolate.evaluate('chin(1,2,3)') == None

def test_populater_ctx():
    Person = {
        "age": 20,
        "name": {
	    "first": "Travis",
	    "last": "Mottershead",
	    "full": "Travis Mottershead"
        }
    }
    ctx			=  Populater( Person )

    assert ctx('{{ age }}') == '20'
    assert ctx('< age') == 20
    assert ctx('= {{ age }}') == 20
    assert ctx("{{ name.first }} {{ name.last }}") == "Travis Mottershead"
    assert ctx("{{ name.first }} {{ name.first }}") == "Travis Travis"
    assert ctx("< name.first")			== "Travis"
    assert ctx("= {{ age }} > 18") is True
    assert ctx("{{ name.none }}")		== ""
    assert ctx("= {{ name.none }}") is None
    assert ctx(":= {{ name.full }}")		== "= Travis Mottershead"
    assert ctx("= '= {{ name.full }}'")		== "= Travis Mottershead"
    assert ctx(":: {{ name.full }}")		== ": Travis Mottershead"
    assert ctx("= self['name']['full']")	== "Travis Mottershead"




from setuptools import setup

setup(
    name                        = "populater",
    packages                    = [
        "populater",
    ],
    package_dir                 = {
        "populater":		".",
    },
    install_requires            = [],
    version                     = "0.1.1",
    include_package_data        = True,
    author                      = "Matthew Brisebois",
    author_email                = "matthew@webheroes.ca",
    url                         = "https://github.com/webheroesinc/populater",
    license                     = "Dual License; GPLv3 and Proprietary",
    description                 = "Cross platform string syntax for reliably extracting object data",
    long_description            = """
Cross platform string syntax for reliably extracting object data

===============
 Usage examples
===============

::

      import Populater
    
      Person = {
          "age": 17,
          "name": {
              "first": "Marty",
              "last": "Mcfly",
              "full": "Marty Mcfly"
          }
      }
      ctx				=  Populater( Person )
    
      assert ctx('{{ age }}')			== '17'
      assert ctx('< age')			== 17
      assert ctx('= {{ age }}')			== 17
      assert ctx("< name.first")		== "Marty"
      assert ctx("= {{ age }} > 18")		is True
      assert ctx("{{ name.none }}")		== ""
      assert ctx("= {{ name.none }}")		is None
      assert ctx(":= {{ name.full }}")		== "= Marty Mcfly"
      assert ctx("= '= {{ name.full }}'")	== "= Marty Mcfly"
      assert ctx(":: {{ name.full }}")		== ": Marty Mcfly"
      assert ctx("= self['name']['full']")	== "Marty Mcfly"

    """,
    keywords                    = [""],
    classifiers                 = [
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "License :: Other/Proprietary License",
        "Programming Language :: Python :: 3.5",
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers"
    ],
)
